from manifest_edit import BasePlugin
from manifest_edit import ManifestIterator
import libfmp4
from schema import Schema, Or


class Plugin(BasePlugin):
    '''
    supplemental_property plugin.

    The purpose of this plugin is to add a supplemental property with the
    configured schemeIdUri and Value as a SupplementalProperty, in the
    position specified in the configuration file.
    '''

    _schema = Schema(
        {
            'supplementalPropertyName': str,
            'supplementalPropertyValue': Or(str, int)
        }
    )

    def validate_config(self, manifest_format):
        self.manifest_iterator = ManifestIterator(
            self.config, plugin_config_schema=self._schema)
        return self.manifest_iterator.is_valid()

    def addSupplementalProperty(self, manifest):

        for suppl_property_config, element in self.manifest_iterator(
                    manifest,
                    ['supplementalPropertyName', 'supplementalPropertyValue']
                ):
            suppl_property = libfmp4.mpd.descriptor(
                suppl_property_config['supplementalPropertyName'],
                suppl_property_config['supplementalPropertyValue']
            )
            element.supplemental_properties.append(suppl_property)

    def mpd(self, manifest):
        self.addSupplementalProperty(manifest)
